@@ -42,6 +42,7 @@
 #define DPKG_ZSTD_MAX_LEVEL 22
 #define ZSTD_CLEVEL_DEFAULT 3
 #endif
+#define DPKG_ZSTD_CLEVEL_DEFAULT 19
 #ifdef WITH_LIBBZ2
 #include <bzlib.h>
 #endif
@@ -132,6 +133,7 @@ struct compressor {
 	const char *name;
 	const char *extension;
 	int default_level;
+
 	void (*fixup_params)(struct compress_params *params);
 	void (*compress)(struct compress_params *params,
 	                 int fd_in, int fd_out, const char *desc);
@@ -215,8 +217,8 @@ decompress_gzip(struct compress_params *params, int fd_in, int fd_out,
 
 			if (z_errnum == Z_ERRNO)
 				errmsg = strerror(errno);
-			ohshit(_("%s: internal gzip read error: '%s'"), desc,
-			       errmsg);
+			ohshit(_("%s: internal gzip read error: '%s'"),
+			       desc, errmsg);
 		}
 		if (actualread == 0) /* EOF. */
 			break;
@@ -287,8 +289,8 @@ compress_gzip(struct compress_params *params, int fd_in, int fd_out,
 
 			if (z_errnum == Z_ERRNO)
 				errmsg = strerror(errno);
-			ohshit(_("%s: internal gzip write error: '%s'"), desc,
-			       errmsg);
+			ohshit(_("%s: internal gzip write error: '%s'"),
+			       desc, errmsg);
 		}
 	}
 
@@ -306,7 +308,10 @@ compress_gzip(struct compress_params *params, int fd_in, int fd_out,
 	}
 }
 #else
-static const char *env_gzip[] = { "GZIP", NULL };
+static const char *env_gzip[] = {
+	"GZIP",
+	NULL,
+};
 
 static void
 decompress_gzip(struct compress_params *params, int fd_in, int fd_out,
@@ -383,8 +388,8 @@ decompress_bzip2(struct compress_params *params, int fd_in, int fd_out,
 
 			if (bz_errnum == BZ_IO_ERROR)
 				errmsg = strerror(errno);
-			ohshit(_("%s: internal bzip2 read error: '%s'"), desc,
-			       errmsg);
+			ohshit(_("%s: internal bzip2 read error: '%s'"),
+			       desc, errmsg);
 		}
 		if (actualread == 0) /* EOF. */
 			break;
@@ -434,8 +439,8 @@ compress_bzip2(struct compress_params *params, int fd_in, int fd_out,
 
 			if (bz_errnum == BZ_IO_ERROR)
 				errmsg = strerror(errno);
-			ohshit(_("%s: internal bzip2 write error: '%s'"), desc,
-			       errmsg);
+			ohshit(_("%s: internal bzip2 write error: '%s'"),
+			       desc, errmsg);
 		}
 	}
 
@@ -447,8 +452,8 @@ compress_bzip2(struct compress_params *params, int fd_in, int fd_out,
 
 		if (bz_errnum == BZ_IO_ERROR)
 			errmsg = strerror(errno);
-		ohshit(_("%s: internal bzip2 write error: '%s'"), desc,
-		       errmsg);
+		ohshit(_("%s: internal bzip2 write error: '%s'"),
+		       desc, errmsg);
 	}
 
 	/* Because BZ2_bzWriteClose has done a fflush on the file handle,
@@ -458,7 +463,11 @@ compress_bzip2(struct compress_params *params, int fd_in, int fd_out,
 		ohshite(_("%s: internal bzip2 write error"), desc);
 }
 #else
-static const char *env_bzip2[] = { "BZIP", "BZIP2", NULL };
+static const char *env_bzip2[] = {
+	"BZIP",
+	"BZIP2",
+	NULL,
+};
 
 static void
 decompress_bzip2(struct compress_params *params, int fd_in, int fd_out,
@@ -616,8 +625,8 @@ filter_lzma(struct io_lzma *io, int fd_in, int fd_out)
 static void DPKG_ATTR_NORET
 filter_lzma_error(struct io_lzma *io, lzma_ret ret)
 {
-	ohshit(_("%s: lzma error: %s"), io->desc,
-	       dpkg_lzma_strerror(io, ret));
+	ohshit(_("%s: lzma error: %s"),
+	       io->desc, dpkg_lzma_strerror(io, ret));
 }
 
 #ifdef HAVE_LZMA_MT_ENCODER
@@ -792,7 +801,11 @@ compress_xz(struct compress_params *params, int fd_in, int fd_out,
 	filter_lzma(&io, fd_in, fd_out);
 }
 #else
-static const char *env_xz[] = { "XZ_DEFAULTS", "XZ_OPT", NULL };
+static const char *env_xz[] = {
+	"XZ_DEFAULTS",
+	"XZ_OPT",
+	NULL,
+};
 
 static void
 decompress_xz(struct compress_params *params, int fd_in, int fd_out,
@@ -802,6 +815,7 @@ decompress_xz(struct compress_params *params, int fd_in, int fd_out,
 	char *threads_opt = NULL;
 
 	command_decompress_init(&cmd, XZ, desc);
+	command_add_arg(&cmd, "--format=xz");
 
 	if (params->threads_max > 0) {
 		threads_opt = str_fmt("-T%d", params->threads_max);
@@ -1288,7 +1302,7 @@ compress_zstd(struct compress_params *params, int fd_in, int fd_out,
 static const struct compressor compressor_zstd = {
 	.name = "zstd",
 	.extension = ".zst",
-	.default_level = 19,
+	.default_level = DPKG_ZSTD_CLEVEL_DEFAULT,
 	.fixup_params = fixup_none_params,
 	.compress = compress_zstd,
 	.decompress = decompress_zstd,
@@ -1382,6 +1396,7 @@ compressor_fixup_params(struct compress_params *params)
 		params->level = compressor(params->type)->default_level;
 }
 
+/* TODO: Refactor compression level and strategy checks into functions. */
 bool
 compressor_check_params(struct compress_params *params, struct dpkg_error *err)
 {
@@ -1422,7 +1437,7 @@ decompress_filter(struct compress_params *params, int fd_in, int fd_out,
 	struct varbuf desc = VARBUF_INIT;
 
 	va_start(args, desc_fmt);
-	varbuf_vprintf(&desc, desc_fmt, args);
+	varbuf_add_vfmt(&desc, desc_fmt, args);
 	va_end(args);
 
 	compressor(params->type)->decompress(params, fd_in, fd_out, desc.buf);
@@ -1438,7 +1453,7 @@ compress_filter(struct compress_params *params, int fd_in, int fd_out,
 	struct varbuf desc = VARBUF_INIT;
 
 	va_start(args, desc_fmt);
-	varbuf_vprintf(&desc, desc_fmt, args);
+	varbuf_add_vfmt(&desc, desc_fmt, args);
 	va_end(args);
 
 	compressor(params->type)->compress(params, fd_in, fd_out, desc.buf);
