@@ -20,8 +20,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-use strict;
-use warnings;
+use v5.36;
 
 use File::Path qw(remove_tree);
 use File::Copy;
@@ -31,6 +30,9 @@ use POSIX qw(:sys_wait_h);
 use Dpkg ();
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
+use Dpkg::SysInfo qw(
+    get_num_processors
+);
 use Dpkg::BuildTypes;
 use Dpkg::BuildOptions;
 use Dpkg::BuildProfiles qw(set_build_profiles);
@@ -189,6 +191,7 @@ my $signchanges = 1;
 my $buildtarget = 'build';
 my $binarytarget = 'binary';
 my $host_arch = '';
+my $host_arch_variant = '';
 my $host_type = '';
 my $target_arch = '';
 my $target_type = '';
@@ -242,17 +245,17 @@ while (@ARGV) {
     $_ = shift @ARGV;
 
     if (/^(?:--help|-\?)$/) {
-	usage;
-	exit 0;
+        usage;
+        exit 0;
     } elsif (/^--version$/) {
-	showversion;
-	exit 0;
+        showversion;
+        exit 0;
     } elsif (/^--admindir$/) {
         $admindir = shift @ARGV;
     } elsif (/^--admindir=(.*)$/) {
-	$admindir = $1;
+        $admindir = $1;
     } elsif (/^--source-option=(.*)$/) {
-	push @source_opts, $1;
+        push @source_opts, $1;
     } elsif (/^--buildinfo-file=(.*)$/) {
         $buildinfo_file = $1;
         usageerr(g_('missing .buildinfo filename')) if not length $buildinfo_file;
@@ -278,79 +281,84 @@ while (@ARGV) {
             push @changes_opts, $changes_opt;
         }
     } elsif (/^--jobs(?:-try)?$/) {
-	$parallel = '';
-	$parallel_force = 0;
+        $parallel = '';
+        $parallel_force = 0;
     } elsif (/^(?:-[jJ]|--jobs(?:-try)?=)(\d*|auto)$/) {
-	$parallel = $1 || '';
-	$parallel_force = 0;
+        $parallel = $1 || '';
+        $parallel_force = 0;
     } elsif (/^--jobs-force(?:=(\d*|auto))?$/) {
         $parallel = $1 || '';
         $parallel_force = 1;
     } elsif (/^(?:-r|--root-command=)(.*)$/) {
-	my $arg = $1;
-	@rootcommand = split ' ', $arg;
+        my $arg = $1;
+        @rootcommand = split ' ', $arg;
     } elsif (/^--check-command=(.*)$/) {
-	$check_command = $1;
+        $check_command = $1;
     } elsif (/^--check-option=(.*)$/) {
-	push @check_opts, $1;
+        push @check_opts, $1;
     } elsif (/^--hook-([^=]+)=(.*)$/) {
-	my ($hook_name, $hook_cmd) = ($1, $2);
-	usageerr(g_('unknown hook name %s'), $hook_name)
-	    if not exists $hook{$hook_name};
-	usageerr(g_('missing hook %s command'), $hook_name)
-	    if not defined $hook_cmd;
-	$hook{$hook_name} = $hook_cmd;
+        my ($hook_name, $hook_cmd) = ($1, $2);
+        usageerr(g_('unknown hook name %s'), $hook_name)
+            if not exists $hook{$hook_name};
+        usageerr(g_('missing hook %s command'), $hook_name)
+            if not defined $hook_cmd;
+        $hook{$hook_name} = $hook_cmd;
     } elsif (/^(--buildinfo-id)=.*$/) {
-	# Deprecated option
-	warning(g_('%s is deprecated; it is without effect'), $1);
+        # Deprecated option.
+        warning(g_('%s is deprecated; it is without effect'), $1);
     } elsif (/^--sign-backend=(.*)$/) {
-	$signbackend = $1;
+        $signbackend = $1;
     } elsif (/^(?:-p|--sign-command=)(.*)$/) {
-	$signcommand = $1;
+        $signcommand = $1;
     } elsif (/^--sign-keyfile=(.*)$/) {
-	$signkeyfile = $1;
+        $signkeyfile = $1;
     } elsif (/^(?:-k|--sign-keyid=|--sign-key=)(.*)$/) {
-	$signkeyid = $1;
+        $signkeyid = $1;
     } elsif (/^--(no-)?check-builddeps$/) {
-	$checkbuilddep = !(defined $1 and $1 eq 'no-');
+        $checkbuilddep = ! (defined $1 and $1 eq 'no-');
     } elsif (/^-([dD])$/) {
-	$checkbuilddep = ($1 eq 'D');
+        $checkbuilddep = ($1 eq 'D');
     } elsif (/^--ignore-builtin-builddeps$/) {
-	$check_builtin_builddep = 0;
+        $check_builtin_builddep = 0;
     } elsif (/^-s(gpg|pgp)$/) {
-	# Deprecated option
-	warning(g_('-s%s is deprecated; always using gpg style interface'), $1);
+        # Deprecated option.
+        warning(g_('-s%s is deprecated; always using gpg style interface'), $1);
     } elsif (/^--force-sign$/) {
-	$signforce = 1;
+        $signforce = 1;
     } elsif (/^--no-sign$/) {
-	$signforce = 0;
-	$signsource = 0;
-	$signbuildinfo = 0;
-	$signchanges = 0;
+        $signforce = 0;
+        $signsource = 0;
+        $signbuildinfo = 0;
+        $signchanges = 0;
     } elsif (/^-us$/ or /^--unsigned-source$/) {
-	$signsource = 0;
+        $signsource = 0;
     } elsif (/^-ui$/ or /^--unsigned-buildinfo$/) {
-	$signbuildinfo = 0;
+        $signbuildinfo = 0;
     } elsif (/^-uc$/ or /^--unsigned-changes$/) {
-	$signbuildinfo = 0;
-	$signchanges = 0;
+        $signbuildinfo = 0;
+        $signchanges = 0;
     } elsif (/^-ap$/ or /^--sign-pausa$/) {
-	$signpause = 1;
+        $signpause = 1;
     } elsif (/^-a$/ or /^--host-arch$/) {
-	$host_arch = shift;
+        $host_arch = shift;
     } elsif (/^-a(.*)$/ or /^--host-arch=(.*)$/) {
-	$host_arch = $1;
+        $host_arch = $1;
+    } elsif (/^--host-arch$-variant/) {
+	$host_arch_variant = shift;
+    } elsif (/^--host-arch-variant=(.*)$/) {
+	$host_arch_variant = $1;
     } elsif (/^-P(.*)$/ or /^--build-profiles=(.*)$/) {
-	my $arg = $1;
-	@build_profiles = split /,/, $arg;
+        my $arg = $1;
+        @build_profiles = split /,/, $arg;
     } elsif (/^-s[iad]$/) {
-	push @changes_opts, $_;
+        push @changes_opts, $_;
     } elsif (/^--(?:compression-level|compression)=.+$/) {
-	push @source_opts, $_;
+        push @source_opts, $_;
     } elsif (/^--(?:diff-ignore|tar-ignore)(?:=.+)?$/) {
-	push @source_opts, $_;
+        push @source_opts, $_;
     } elsif (/^-(?:s[nsAkurKUR]|[zZ].*|i.*|I.*)$/) {
-	push @source_opts, $_; # passed to dpkg-source
+        # Passed to dpkg-source.
+        push @source_opts, $_;
     } elsif (/^-tc$/ or /^--post-clean$/) {
         $postclean = 1;
     } elsif (/^--no-post-clean$/) {
@@ -358,17 +366,19 @@ while (@ARGV) {
     } elsif (/^--sanitize-env$/) {
         $sanitize_env = 1;
     } elsif (/^-t$/ or /^--host-type$/) {
-	$host_type = shift; # Order DOES matter!
+        # Order DOES matter!
+        $host_type = shift;
     } elsif (/^-t(.*)$/ or /^--host-type=(.*)$/) {
-	$host_type = $1; # Order DOES matter!
+        # Order DOES matter!
+        $host_type = $1;
     } elsif (/^--target-arch$/) {
-	$target_arch = shift;
+        $target_arch = shift;
     } elsif (/^--target-arch=(.*)$/) {
-	$target_arch = $1;
+        $target_arch = $1;
     } elsif (/^--target-type$/) {
-	$target_type = shift;
+        $target_type = shift;
     } elsif (/^--target-type=(.*)$/) {
-	$target_type = $1;
+        $target_type = $1;
     } elsif (/^(?:--target|--rules-target|-T)$/) {
         push @call_target, split /,/, shift @ARGV;
     } elsif (/^(?:--target=|--rules-target=|-T)(.+)$/) {
@@ -385,38 +395,38 @@ while (@ARGV) {
     } elsif (/^--build=(.*)$/) {
         set_build_type_from_options($1, $_);
     } elsif (/^-b$/) {
-	set_build_type(BUILD_BINARY, $_);
+        set_build_type(BUILD_BINARY, $_);
     } elsif (/^-B$/) {
-	set_build_type(BUILD_ARCH_DEP, $_);
+        set_build_type(BUILD_ARCH_DEP, $_);
     } elsif (/^-A$/) {
-	set_build_type(BUILD_ARCH_INDEP, $_);
+        set_build_type(BUILD_ARCH_INDEP, $_);
     } elsif (/^-S$/) {
-	set_build_type(BUILD_SOURCE, $_);
+        set_build_type(BUILD_SOURCE, $_);
     } elsif (/^-G$/) {
-	set_build_type(BUILD_SOURCE | BUILD_ARCH_DEP, $_);
+        set_build_type(BUILD_SOURCE | BUILD_ARCH_DEP, $_);
     } elsif (/^-g$/) {
-	set_build_type(BUILD_SOURCE | BUILD_ARCH_INDEP, $_);
+        set_build_type(BUILD_SOURCE | BUILD_ARCH_INDEP, $_);
     } elsif (/^-F$/) {
-	set_build_type(BUILD_FULL, $_);
+        set_build_type(BUILD_FULL, $_);
     } elsif (/^-v(.*)$/) {
-	$since = $1;
+        $since = $1;
     } elsif (/^-m(.*)$/ or /^--(?:source|build)-by=(.*)$/) {
-	$maint = $1;
+        $maint = $1;
     } elsif (/^-e(.*)$/ or /^--(?:changed|release)-by=(.*)$/) {
-	$changedby = $1;
+        $changedby = $1;
     } elsif (/^-C(.*)$/) {
-	$desc = $1;
+        $desc = $1;
     } elsif (m/^-[EW]$/) {
-	# Deprecated option
-	warning(g_('%s is deprecated; it is without effect'), $_);
+        # Deprecated option.
+        warning(g_('%s is deprecated; it is without effect'), $_);
     } elsif (/^-R(.*)$/ or /^--rules-file=(.*)$/) {
-	my $arg = $1;
-	@debian_rules = split ' ', $arg;
+        my $arg = $1;
+        @debian_rules = split ' ', $arg;
     } elsif ($_ eq '--') {
         $source = shift @ARGV;
         last;
     } elsif (/^-/) {
-	usageerr(g_('unknown option or argument %s'), $_);
+        usageerr(g_('unknown option or argument %s'), $_);
     } else {
         $source = $_;
         last;
@@ -425,7 +435,7 @@ while (@ARGV) {
 
 if (@call_target) {
     my $targets = join ',', @call_target;
-    set_build_type_from_targets($targets, '--rules-target', nocheck => 1);
+    set_build_type_from_targets($targets, '--rules-target', no_check => 1);
 }
 
 if (build_has_all(BUILD_BINARY)) {
@@ -440,9 +450,9 @@ if (build_has_all(BUILD_BINARY)) {
 }
 
 if (not $preclean) {
-    # -nc without -b/-B/-A/-S/-F implies -b
+    # -nc without -b/-B/-A/-S/-F implies -b.
     set_build_type(BUILD_BINARY) if build_has_any(BUILD_DEFAULT);
-    # -nc with -S implies no dependency checks
+    # -nc with -S implies no dependency checks.
     $checkbuilddep = 0 if build_is(BUILD_SOURCE);
 }
 
@@ -472,14 +482,7 @@ run_hook('preinit');
 
 if (defined $parallel) {
     if ($parallel eq 'auto') {
-        # Most Unices.
-        $parallel = qx(getconf _NPROCESSORS_ONLN 2>/dev/null);
-        # Fallback for at least Irix.
-        $parallel = qx(getconf _NPROC_ONLN 2>/dev/null) if $?;
-        # Fallback to serial execution if cannot infer the number of online
-        # processors.
-        $parallel = '1' if $?;
-        chomp $parallel;
+        $parallel = get_num_processors();
     }
     if ($parallel_force) {
         $ENV{MAKEFLAGS} //= '';
@@ -575,6 +578,8 @@ push @arch_opts, ('--host-type', $host_type) if $host_type;
 push @arch_opts, ('--target-arch', $target_arch) if $target_arch;
 push @arch_opts, ('--target-type', $target_type) if $target_type;
 
+$ENV{DEB_HOST_ARCH_VARIANT} = $host_arch_variant if $host_arch_variant;
+
 open my $arch_env, '-|', 'dpkg-architecture', '-f', @arch_opts
     or subprocerr('dpkg-architecture');
 while (<$arch_env>) {
@@ -584,9 +589,11 @@ while (<$arch_env>) {
 }
 close $arch_env or subprocerr('dpkg-architecture');
 
+# this is now the architecture variant, if set
 my $arch;
 if (build_has_any(BUILD_ARCH_DEP)) {
-    $arch = mustsetvar($ENV{DEB_HOST_ARCH}, g_('host architecture'));
+	$arch = $ENV{DEB_HOST_ARCH_VARIANT};
+	$arch = mustsetvar($ENV{DEB_HOST_ARCH}, g_('host architecture')) if (not $arch);
 } elsif (build_has_any(BUILD_ARCH_INDEP)) {
     $arch = 'all';
 } elsif (build_has_any(BUILD_SOURCE)) {
@@ -656,7 +663,7 @@ my $build_driver = Dpkg::BuildDriver->new(
 );
 
 #
-# Preparation of environment stops here
+# Preparation of environment stops here.
 #
 
 run_hook('init');
@@ -679,9 +686,9 @@ if ($checkbuilddep) {
     if (not WIFEXITED($?)) {
         subprocerr('dpkg-checkbuilddeps');
     } elsif (WEXITSTATUS($?)) {
-	warning(g_('build dependencies/conflicts unsatisfied; aborting'));
-	warning(g_('(Use -d flag to override.)'));
-	exit 3;
+        errormsg(g_('build dependencies/conflicts unsatisfied; aborting'));
+        hint(g_('satisfy build dependencies with your package manager frontend'));
+        exit 3;
     }
 }
 
@@ -749,6 +756,9 @@ $elf_package_metadata .= ",\"os\":\"$os_id\"";
 $elf_package_metadata .= ",\"name\":\"$pkg\"";
 $elf_package_metadata .= ",\"version\":\"$version\"";
 $elf_package_metadata .= ",\"architecture\":\"$arch\"";
+if (defined $ENV{DEB_HOST_ARCH_VARIANT}) {
+    $elf_package_metadata .= ",\"architectureVariant\":\"$ENV{DEB_HOST_ARCH_VARIANT}\"";
+}
 if (defined $ENV{DEB_BUILD_DEBUG_INFO_URL}) {
     $elf_package_metadata .= ",\"debugInfoUrl\":\"$ENV{DEB_BUILD_DEBUG_INFO_URL}\"";
 }
@@ -854,7 +864,10 @@ if ($signsource) {
     $buildinfo->load($buildinfo_file);
     my $checksums = Dpkg::Checksums->new();
     $checksums->add_from_control($buildinfo);
-    $checksums->add_from_file("../$pv.dsc", update => 1, key => "$pv.dsc");
+    $checksums->add_from_file("../$pv.dsc",
+        update => 1,
+        key => "$pv.dsc",
+    );
     $checksums->export_to_control($buildinfo);
     $buildinfo->save($buildinfo_file);
 }
@@ -896,7 +909,7 @@ sub mustsetvar {
     my ($var, $text) = @_;
 
     error(g_('unable to determine %s'), $text)
-	unless defined($var);
+        unless defined($var);
 
     info("$text $var");
     return $var;
@@ -1002,7 +1015,7 @@ sub describe_build {
     my $ext = compression_get_file_extension_regex();
 
     if (fileomitted($files, qr/\.deb/)) {
-        # source-only upload
+        # source-only upload.
         if (fileomitted($files, qr/\.diff\.$ext/) and
             fileomitted($files, qr/\.debian\.tar\.$ext/)) {
             return g_('source-only upload: Debian-native package');
